
!GRPackage class methodsFor: '*seaside-flow' stamp: 'lr 2/17/2010 15:12'!
seasideFlow
	^ self new
		name: 'Seaside-Flow';
		description: 'Define control flow as a continuous piece of code, with natural, linear control flow.';
		addDependency: 'Seaside-Core'; "Methods are added to WARequestContext"
		addDependency: 'Seaside-Component'; "Methods are added to WAComponent"
		addDependency: 'Seaside-RenderLoop'; "Methods are added to WASession that depend on RenderLoop"
		addDependency: 'Seaside-Tools-Core'; "We currently us WAFlowMain class>>initialize to configure the application defaults to use our Main and ActionPhaseContinuation"
		url: #seasideUrl;
		yourself! !

!WARequestContext methodsFor: '*seaside-flow' stamp: 'jf 1/21/2009 10:51'!
redirect
	^ self respondAndContinue: [ :url | self redirectTo: url ]! !

!WARequestContext methodsFor: '*seaside-flow' stamp: 'lr 2/20/2009 11:06'!
redirectWithMessage: aString delay: aNumber 
	self respondAndContinue: [ :url | 
		self responseGenerator 
			refreshAfter: aNumber
			location: url
			message: aString ]! !

!WARequestContext methodsFor: '*seaside-flow' stamp: 'pmm 1/28/2011 20:45'!
respondAndContinue: aBlock 
	^ GRPlatform current seasideSuspendFlowDo: [ :cc | 
		aBlock value: (self session actionUrlForContinuation: cc).
		self respond ]! !

!WASession methodsFor: '*seaside-flow' stamp: 'jf 9/30/2009 01:02'!
respond: aBlock 
	self 
		greaseDeprecatedApi: 'WASession>>#respond:'
		details: 'Use WARequestContext>>#respondAndContinue: (requires the Seaside-Flow package).'.
	^ self requestContext respondAndContinue: aBlock! !

!WAPainterVisitor methodsFor: '*seaside-flow' stamp: 'jf 2/8/2009 10:52'!
visitTask: aTask
	self visitComponent: aTask! !

!WAComponent methodsFor: '*seaside-flow-filters' stamp: 'lr 8/31/2008 14:30'!
authenticateWith: anAuthenticator during: aBlock
	"Authenticate calls during the evaluation of aBlock."
	
	self 
		filterWith: (WAAuthenticationFilter new
			authenticator: anAuthenticator;
			yourself)
		during: aBlock! !

!WAComponent methodsFor: '*seaside-flow' stamp: 'jf 12/19/2009 01:43'!
call: aComponent 
	"Pass control from the receiver to aComponent. The receiver will be temporarily replaced with aComponent. Code can return from here later on by sending #answer: to aComponent."

	^ self wait: [ :cc | self call: aComponent onAnswer: cc ]! !

!WAComponent methodsFor: '*seaside-flow-dialog' stamp: 'lr 1/24/2009 21:03'!
chooseFrom: aCollection
	"Choose an item from the given aCollection. Answer the selected item."

	^ self wait: [ :cc | self chooseFrom: aCollection onAnswer: cc ]! !

!WAComponent methodsFor: '*seaside-flow-dialog' stamp: 'lr 1/24/2009 21:04'!
chooseFrom: aCollection caption: aString
	"Choose an item from the given aCollection with caption aString. Answer the selected item."
	
	^ self wait: [ :cc | self chooseFrom: aCollection caption: aString onAnswer: cc ]! !

!WAComponent methodsFor: '*seaside-flow-dialog' stamp: 'lr 8/3/2009 20:23'!
chooseFrom: aCollection default: anObject
	"Choose an item from the given aCollection and the default anObject. Answer the selected item."

	^ self wait: [ :cc | self chooseFrom: aCollection default: anObject onAnswer: cc ]! !

!WAComponent methodsFor: '*seaside-flow-dialog' stamp: 'lr 8/3/2009 20:24'!
chooseFrom: aCollection default: anObject caption: aString
	"Choose an item from the given aCollection, the default anObject and the caption aString. Answer the selected item."
	
	^ self wait: [ :cc | self chooseFrom: aCollection default: anObject caption: aString onAnswer: cc ]! !

!WAComponent methodsFor: '*seaside-flow-dialog' stamp: 'lr 1/24/2009 21:04'!
confirm: aString
	"Display a yes/no dialog with caption aString. Answer true if the response is yes, false if no."

	^ self wait: [ :cc | self confirm: aString onAnswer: cc ]! !

!WAComponent methodsFor: '*seaside-flow-filters' stamp: 'lr 8/31/2008 14:12'!
filterWith: aFilter during: aBlock
	| answer |
	self session addFilter: aFilter.
	answer := aBlock value.
	self session removeFilter: aFilter.
	^ answer! !

!WAComponent methodsFor: '*seaside-flow-dialog' stamp: 'lr 1/24/2009 21:04'!
inform: aString
	"Display a dialog with aString to the user until he clicks the ok button."

	^ self wait: [ :cc | self inform: aString onAnswer: cc ]! !

!WAComponent methodsFor: '*seaside-flow-dialog' stamp: 'lr 1/24/2009 21:05'!
request: aRequestString
	"Display an input dialog with the question aRequestString. Answer the string the user enters."

	^ self wait: [ :cc | self request: aRequestString onAnswer: cc ]! !

!WAComponent methodsFor: '*seaside-flow-dialog' stamp: 'lr 1/24/2009 21:05'!
request: aRequestString default: aDefaultString
	"Display an input dialog with the question aRequestString and the default string aDefaultString. Answer the string the user enters."
	
	^ self wait: [ :cc | self request: aRequestString default: aDefaultString onAnswer: cc ]! !

!WAComponent methodsFor: '*seaside-flow-dialog' stamp: 'lr 1/24/2009 21:05'!
request: aRequestString label: aLabelString
	"Display an input dialog with the question aRequestString and the button label aLabelString. Answer the string the user enters."
	
	^ self wait: [ :cc | self request: aRequestString label: aLabelString onAnswer: cc ]! !

!WAComponent methodsFor: '*seaside-flow-dialog' stamp: 'lr 1/24/2009 21:05'!
request: aRequestString label: aLabelString default: aDefaultString
	"Display an input dialog with the question aRequestString, the button label aLabelString and the default string aDefaultString. Answer the string the user enters."

	^ self wait: [ :cc | self request: aRequestString label: aLabelString default: aDefaultString onAnswer: cc ]! !

!WAComponent methodsFor: '*seaside-flow' stamp: 'jf 2/9/2010 01:31'!
wait: aBlock 
	"Evaluate aBlock and pass in a continuation that can be evaluated to answer to the place where this very method was called."
	^ GRPlatform current seasideSuspendFlowDo: [ :cc | aBlock value: cc ]! !

!GRPlatform methodsFor: '*seaside-flow' stamp: 'jf 2/9/2010 01:31'!
seasideSuspendFlowDo: aBlock
	"Evaluate aBlock, passing in a continuation that can be evaluated to answer to the place where this very method was called. This method is presumed to be executed only within the scope of an executing Callback or Task and should, ideally, create a continuation that includes only the contexts that make up the Callback or Task execution (a partial or delimited continuation). Do not use this method, if you don't understand this comment."
	self subclassResponsibility! !
